let data;
let neoVals;
let notNeoVals;
let AXIS_INSET;
let AXIS_WIDTH;
let AXIS_HEIGHT;
let black;
let white;
let red;
let grey;

function preload() {
  data = loadTable("https://gist.githubusercontent.com/colmpat/44cec6db3a9f84ffb001e3fc0166eba6/raw/079a6540637fae9ed3dc2049d0254a42233be201/sb_a_sorted.csv", "csv", "header");
}

function setup() {
  createCanvas(700, 600);
  angleMode(DEGREES);
  rectMode(CENTER);
  AXIS_INSET = 90;
  AXIS_WIDTH = width - 2 * AXIS_INSET;
  AXIS_HEIGHT = height - AXIS_INSET - 60;
  black = color(25, 50, 60);
  white = color(243, 247, 240);
  red = color(169, 63, 85);
  grey = color(110, 136, 148);

  neoVals = [];
  notNeoVals = [];

  data.getRows().forEach(row => {
    if(row.get("neo") === "Y") {
      neoVals.push(parseFloat(row.get("a")));
    } else {
      notNeoVals.push(parseFloat(row.get("a")));
    }
  });
}

function draw() {
  background(white);

  drawBoxes(AXIS_INSET);
  title("Average Distance from the Sun for Small Body Asteroids (in au)", 20);
  drawAxis(AXIS_INSET);
  labelAxis("Near Earth Object", "Average Distance from Sun (in au)", AXIS_INSET);
  // drawBars(8, AXIS_INSET);
  noLoop();
}

function drawBoxes(inset) {
  let w = 70;
  let spacing = (AXIS_WIDTH - 2*w) / 3;
  let neoVals = getBoxPlotVals(true);
  let notNeoVals = getBoxPlotVals(false);
  let t1 = createVector(inset + spacing, 0 - inset);
  let t2 = createVector(t1.x + w + spacing, t1.y);

  textAlign(CENTER, TOP); textSize(16);
  drawBox(neoVals, w, t1);
  text("NEO", t1.x + w/2, height - inset + 10);
  drawBox(notNeoVals, w, t2);
  text("Non NEO", t2.x + w/2, height - inset + 10);
}

function drawBox(vals, w, t) {
  textSize(14);
  push();

  translate(t.x, t.y);
  //vals format: [min, q1, med, q3, max]
  let valLabels = vals.map(x => round(x, 4));
  vals = vals.map(x => getHeight(x, vals[4]));

  textAlign(LEFT, CENTER); strokeWeight(1);
  line(w / 2, height - vals[0], w / 2, height - vals[1]);
  text(valLabels[0], w/2 + 8, height - vals[0]);
  line(w / 2, height - vals[4], w / 2, height - vals[3]);
  text(valLabels[4], w/2 + 8, height - vals[4]);

  strokeWeight(2);

  //veretical lines
  line(0, height - vals[1], 0, height - vals[3]);
  line(w, height - vals[1], w, height - vals[3]);

  

  for(let i = 1; i <= 3; i++) {
    strokeWeight(i === 2 ? 3 : 2)
    let val = vals[i];
    line(0, height - val, w, height - val);
    text(valLabels[i], w + 8, height - val);
  }
  pop();
}

function title(title, inset) {
  textSize(22); fill(black); textAlign(LEFT, TOP);
  text(title, inset, inset);
}

function drawAxis(inset) {
  stroke(black); strokeWeight(3);
  //top left to bottom left
  line(inset, 60, inset, height - inset);
  //bottom left to bottom right
  line(inset, height - inset, width - inset, height - inset);
}

function labelAxis(xLabel, yLabel, inset) {
  textAlign(CENTER, CENTER); textSize(18); noStroke();
  //label x axis
  text(xLabel, width / 2, height - 20);

  //translate and rotate vertically
  translate(20, height / 2); rotate(-90);
  //label y axis at the origin of translated grid
  text(yLabel, 20, 0);
  resetMatrix();
  textAlign(RIGHT, CENTER); textSize(14);
  for(let i = 0; i <= 4; i += 0.5) {
    noStroke();
    let h = height - inset - getHeight(i, 4);
    text(i, inset - 10, h);
    stroke(black); strokeWeight(2);
    line(inset - 7, h, inset, h);
  }
}

function getHeight(originalHeight, max) {
  let maxTick = ceil(max);
  console.log(maxTick);
  return (originalHeight * AXIS_HEIGHT) / maxTick; //from ratio: orig / maxTick = new / axis height
}

function getBoxPlotVals(neo) {
  let arr = (neo ? neoVals : notNeoVals);
  let numRows = arr.length;
  //result format:
  //min, q1, med, q3, max
  let result = [];

  if(numRows % 2 === 0) {
    let absoluteMid = numRows / 2;
    result[2] = findMed(0, numRows, neo);
    result[1] = findMed(0, ceil(absoluteMid), neo);
    result[3] = findMed(ceil(absoluteMid), numRows, neo);

  } else {
    let absoluteMid = numRows / 2;
    result[2] = findMed(0, numRows, neo);
    result[1] = findMed(0, floor(absoluteMid), neo);
    result[3] = findMed(ceil(absoluteMid), numRows, neo);
  }

  let iqr = result[3] - result[1];

  let minBoundry = parseFloat(result[1]) - iqr * 1.5;
  let maxBoundry = parseFloat(result[3]) + iqr * 1.5;

  let i = 0;
  while(arr[i] < minBoundry) {
    i++;
  }
  result[0] = arr[i];

  i = numRows - 1;
  while(arr[i] > maxBoundry) {
    i--;
  }
  result[4] = arr[i];

  return result;
}

function findMed(start, end, neo) {
  let len = end - start;
  let arr = (neo ? neoVals : notNeoVals);

  if(len % 2 === 0) {
    let i1 = start + floor((len - 1) / 2);
    let i2 = start + floor((len) / 2);

    return (arr[i1] + arr[i2]) / 2;
  } else {
    let i = start + floor((len) / 2);
    return arr[i];
  }
}
// 0 1 2 3 4
// med = 2
// q1 = 0.5
// q3 = 3.5